/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package librarymanagementsystem;

import java.util.ArrayList;

/**
 * Represents a book's identification including title, author, year, publisher,
 * publisherDate, edition, isbn, numCopies, numCheckedCopies, currentHolders, 
 * idNum, and price.
 * @author Pawprint Studios
 */
public class Book 
{
    
    public String title;
    public String author;
    public int year;
    public String publisher;
    public String publisherDate;
    public int edition;
    public long isbn;
    public int numCopies;
    public int numCheckedCopies;
    private long idNum;
    private double price;
    private ArrayList<Long> currentHolders;
    
    // Ryan Reminder: We need to add more constructors to this class that 
    // require less parameters, i.e. one constructor only requires the Title and
    // the Author.

    /**
     * Represents the title of the book.
     * @param bookTitle represents the title of the book
     */
    public Book(String bookTitle)
    {
        title = bookTitle;
    }

    /**
     * Represents the book's title, author, year, number of copies, number of 
     * checked out copies, current holders, and Library Id number.
     * @param bookTitle represents the title of the book.
     * @param bookAuthor represents the author of the book.
     * @param bookYear represents the year the book was published.
     * @param bookNumCopies represents the number of copies library holds of
     * book.
     * @param bookNumCheckedCopies represents the number of checked out copies 
     * of a book.
     * @param bookIdNum represents the book's ID number.
     * @param bookCurrentHolders represents the current holders of a book.
     */
    public Book(String bookTitle, String bookAuthor, int bookYear, 
        int bookNumCopies, int bookNumCheckedCopies, long bookIdNum, 
        ArrayList<Long> bookCurrentHolders)
    {
        title = bookTitle;
        author = bookAuthor;
        year = bookYear;
        numCopies = bookNumCopies;
        numCheckedCopies = bookNumCheckedCopies;
        idNum = bookIdNum;
        currentHolders = bookCurrentHolders;
    }
    
    
    
    /**
     * Book information containing book's title, author, released year, 
     * publisher, edition, Isbn, number of copies, number of checked copies,
     * who all is has a hold on the book, book's Id number, price. 
     * @param bookTitle this represents the title of the book
     * @param bookAuthor represents the author of the book.
     * @param bookYear represents the year the book was published.
     * @param bookPublisher represents the publisher of the book.
     * @param bookPublisherDate represents date the book was published.
     * @param bookEdition represents the edition of the book.
     * @param bookIsbn represents the ISBN of the book.
     * @param bookNumCopies represents the number of copies of a book within 
     * library.
     * @param bookNumCheckedCopies represents the number of checked out copies 
     * of a book.
     * @param bookIdNum represents the book's ID number.
     * @param bookPrice represents the price of the book.
     * @param bookCurrentHolders represents the current holders of a book.
    */
    public Book(String bookTitle, String bookAuthor, int bookYear, 
        String bookPublisher, String bookPublisherDate, int bookEdition, 
        long bookIsbn, int bookNumCopies, int bookNumCheckedCopies, 
        long bookIdNum, double bookPrice, ArrayList<Long> bookCurrentHolders)
    {
        title = bookTitle;
        author = bookAuthor;
        year = bookYear;
        publisher = bookPublisher;
        publisherDate = bookPublisherDate;
        edition = bookEdition;
        isbn = bookIsbn;
        numCopies = bookNumCopies;
        numCheckedCopies = bookNumCheckedCopies;
        idNum = bookIdNum;
        price = bookPrice;
        currentHolders = bookCurrentHolders;
    }
    
    
    
    //  *** Accessors. ***
    
    public String getTitle()
    {
        return title;
    }
    
    public String getAuthor()
    {
        return author;
    }
    
    public int getYear()
    {
        return year;
    }
    
    public String getPublisher()
    {
        return publisher;
    }
    
    public String getPublisherDate()
    {
        return publisherDate;
    }
    
    public int getEdition()
    {
        return edition;
    }
    
    public long getIsbn()
    {
        return edition;
    }
    
    public int getNumCopies()
    {
        return numCopies;
    }
    
    public int getNumCheckedCopies()
    {
        return numCheckedCopies;
    }
    
    public long getIdNum()
    {
        return idNum;
    }
    
    public double getPrice()
    {
        return price;
    }
    
    public String getCurrentHolders()
    {
        String holders = "";
        if (numCheckedCopies > 0)
        {
            for (int i = 0; i < numCheckedCopies; i++)
            {
                holders += currentHolders.get(i);
                if (i != numCheckedCopies - 1)
                {
                    holders += ",";
                }
            }
        }
        else
        {
            holders += currentHolders.get(0);
        }
        return holders;
    }
    
    
    
    //  *** Other Methods. ***
        
    public void loanBook(long userIdNum)
    {
        currentHolders.add(userIdNum);
        numCheckedCopies++;
    }
     
    
    public void returnBook(long userIdNum)
    {
        currentHolders.remove(userIdNum);
        numCheckedCopies--;
    }
    
    
    
    //  *** Other Methods. ***
    
    /**
     * Compares one book to another.
     * @param otherBook represents other book
     * @return result
     */
    public boolean equalsBook(String otherBook)
    {
        boolean result = false;
        
        if (otherBook != null)
        {
            result = title.equalsIgnoreCase(otherBook);
        }
        
        return result;
    }
    
    
    /**
     * Prints the book and identifications.
     * @return books identifications
     */
    public String printLoan()
    {
        return "Title: " + title + "\nAuthor: " + author + "\nYear: " + year 
            + "\nPrice: " + price + "\nID Number: " + idNum + "\n";
    }
    
    public String printBook()
    {
        return "Title: " + title + "\nAuthor: " + author + "\nYear: " + year 
            + "\nNumber of Copies Available: " + (numCopies - numCheckedCopies) 
            + " out of " + numCopies + "\nID Number: " + idNum + "\n";
    }
    

    public String printFullBook()
    {
        return "Title: " + title + "\nAuthor: " + author + "\nYear: " + year
            + "\nPublisher: " + publisher  + "\nPublication Date: " 
            + publisherDate + "\nEdition: " + edition + "\nISBN: " + isbn
            + "\nNumber of Copies: " + numCopies 
            + "\nNumber of Copies Checked: " + numCheckedCopies 
            + "\nCurrent Holders: " + currentHolders + "\nID Number: " + idNum
            + "\nPrice: " + price;
    }

    
    //It's returning an object for current holder instead of username.
    /**
     * Prints the string version of book and all it's identifications.
     * @return book's identifications. 
     */
    @Override
    public String toString()
    {
        String formattedIdNum = String.format("%06d", idNum);
        String toString = "\n" + title + "," + author + "," + year + "," 
            + publisher + "," + publisherDate + "," + edition + "," + isbn 
            + "," + numCopies + "," + numCheckedCopies + "," + formattedIdNum  
            + "," + price;
        
        if (numCheckedCopies > 0)
        {
            for(Long userId : currentHolders)
            {
                toString += "," + userId;
            }
        }
            
        return toString;
    }
}