/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package librarymanagementsystem;

import java.util.ArrayList;

/**
 * Represents the basic privileges for every user (Patron, Librarian, Admin).
 * @author Pawprint Studios
 */
public class GeneralUser
{
    public String username;
    protected String password;
    protected long userId;
    protected String userType;
    
    
    protected ArrayList<Long> currentLoans;
        
    private String patronName;
    private String email;
    private long phoneNumber;
    private int numLoans;

    
    
    /**
     * 
     */
    public GeneralUser()
    {
        // Ryan Note: I am not sure if I need to put anything in this default
        // constructor.
    }
    
    
    /**
     * General information of user, including username, password, userId, 
     * and user type (Patron, Librarian, Admin).
     * @param personUsername represents the user's username.
     * @param personPassword represents the user's password.
     * @param personUserId represents the user's ID.
     * @param personUserType represents the user's type.
     */
    public GeneralUser(String personUsername, String personPassword, 
        long personUserId, String personUserType)
    {
        username = personUsername;
        password = personPassword;
        userId = personUserId;
        userType = personUserType;
    }
    
    
    public GeneralUser(String personUsername, String personPassword, 
        long personUserId, String personUserType, String name, 
        String patronEmail, long patronPhoneNumber)
    {
        username = personUsername;
        password = personPassword;
        userId = personUserId;
        userType = personUserType;
        patronName = name;
        email = patronEmail;
        phoneNumber = patronPhoneNumber;
    }
    
    
    public GeneralUser(String personUsername, String personPassword, 
        long personUserId, String personUserType, String name, 
        String patronEmail, long patronPhoneNumber, int patronNumLoans)
    {
        username = personUsername;
        password = personPassword;
        userId = personUserId;
        userType = personUserType;
        patronName = name;
        email = patronEmail;
        phoneNumber = patronPhoneNumber;
        numLoans = patronNumLoans;
    }
    
    
    public GeneralUser(String personUsername, String personPassword, 
        long personUserId, String personUserType, String name, 
        String patronEmail, long patronPhoneNumber, int patronNumLoans, 
        ArrayList<Long> patronCurrentLoans)
    {
        username = personUsername;
        password = personPassword;
        userId = personUserId;
        userType = personUserType;
        patronName = name;
        email = patronEmail;
        phoneNumber = patronPhoneNumber;
        numLoans = patronNumLoans;
        currentLoans = patronCurrentLoans;
    }
    
    
    
    //  *** Accessors. ***
    
    /**
     * Gets username.
     * @return username
     */
    public String getUsername()
    {
        return this.username;
    }
    
    
    /**
     * Gets password.
     * @return password
     */
    public String getPassword()
    {
        return this.password;
    }


    /**
     * Gets user ID.
     * @return userId
     */
    public long getUserId()
    {
        return this.userId;
    }
    
    
    /**
     * Gets the user type.
     * @return userType
     */
    public String getUserType()
    {
        return this.userType;
    }
    
    
    public ArrayList<Long> getCurrentLoans()
    {
        return this.currentLoans;
    }
    
    
    public String getPatronName()
    {
        return this.patronName;
    }
    
    
    public String getEmail()
    {
        return this.email;
    }
    
    
    public long getPhoneNumber()
    {
        return this.phoneNumber;
    }
    
    
    public int getNumLoans()
    {
        return this.numLoans;
    }
    
    
    
    //  *** Mutators. ***
    
    /**
     * Sets username.
     * @param nameUser represents the current username.
     */
    public void setUsername(String nameUser)
    {
        this.username = nameUser;
    }
        
    
    /**
     * Sets password
     * @param wordPass represents if passwords is validated.
     */
    public void setPassword(String wordPass)
    {
        this.password = wordPass;
    }
    
    
    public void setCurrentLoans(ArrayList<Long> patronsCurrentLoans)
    {
        currentLoans = patronsCurrentLoans;
    }
    
    
    public void setPatronName(String name)
    {
        patronName = name;
    }
    
    
    public void setEmail(String patronsEmail)
    {
        email = patronsEmail;
    }
    
    
    public void setPhoneNumber(long patronsPhoneNumber)
    {
        phoneNumber = patronsPhoneNumber;
    }
    
    
    public void setNumLoans(int patronsNumLoans)
    {
        numLoans = patronsNumLoans;
    }
    


    // *** Book Stuff. ***
    
    public void loanBook(long bookIdNum)
    {
        currentLoans.add(bookIdNum);
        numLoans++;
    }
    
    
    public void returnBook(long bookIdNum)
    {
        
        currentLoans.remove(bookIdNum);
        numLoans--;
    }


    
    //  *** Other Methods. ***
    
    /**
     * Determines if users match.
     * @param obj represents other username.
     * @return result
     */
    public boolean equals(Object obj)
    {
        boolean result = false;
        
        if (obj != null && getClass() == obj.getClass())
        {
            GeneralUser other = (GeneralUser) obj;
            result = username.equalsIgnoreCase(other.username) 
                && password.equals(other.password) 
                && userId == other.userId 
                && userType.equals(other.userType);
        }
        
        return result;
        
    }
    
    
        public String printUser()
    {
        String formattedIdNum = String.format("%06d", userId);

        String printUser = "Username: " + username + "\nID Number: " 
            + formattedIdNum + "\nUser Type: " + userType 
            + "\nFirst and Last Name: " + patronName;
        
        if (numLoans > 0)
        {
            printUser += "/n" + numLoans + " Current Loans: ";
            for(Long bookId : currentLoans)
            {
                printUser += "," + bookId;
            }
        }
        return printUser;
    }
    
    
        public String printFullUser()
    {
        String formattedIdNum = String.format("%06d", userId);

        String printUser = "Username: " + username + "\nPassword: " + password 
            + "\nID Number: " + formattedIdNum + "\nUser Type: " + userType  
            + "\nFirst and Last Name: " + patronName + "\nEmail Address: " 
            + email + "\nPhone Number: " + phoneNumber + "\nNumber of Loans: " 
            + numLoans;
        
        if (numLoans > 0)
        {
            printUser += "/nCurrent Loans: ";
            for(Long bookId : currentLoans)
            {
                printUser += "," + bookId;
            }
        }
        return printUser;
    }
    
    
    // Ryan Note: Update this later at some point
    /**
     * Prints the user's name and password.
     * @return username and password
     */
    @Override
    public String toString()
    {
        String formattedIdNum = String.format("%06d", userId);
        String toString = "\n" + username + "," + password + "," + formattedIdNum 
            + "," + userType + "," + patronName + "," + email + ","  
            + phoneNumber + "," + numLoans;
        
        if (numLoans > 0)
        {
            for(Long bookId : currentLoans)
            {
                toString += "," + bookId;
            }
        }
            
        return toString;
    }
}
